/***************************************************************************

  Mini-Pac Copyright (C) 1999 by Michael Balfour.  All Rights Reserved.


  Pac-Man Copyright (C) 1980 by Namco, Ltd.  All Rights Reserved.


  Multi-Z80 32 Bit emulator 
  Copyright (C) 1996-1999 Neil Bradley, All rights reserved
  Author      : Neil Bradley (neil@synthcom.com)
  Distribution: ftp://ftp.synthcom.com/pub/emulators/cpu/makez80.zip (latest)

 ***************************************************************************/

/* Visual C define for smaller, faster header files */
#define WIN32_LEAN_AND_MEAN  

#include <windows.h>         // include the standard windows stuff
#include <windowsx.h>        // include the 32 bit stuff
#include <mmsystem.h>        // include the multi media stuff
							 // note you need winmm.lib also

#include <ddraw.h>           // include direct draw components
#include <dsound.h>			 // include direct sound components

#include <stdlib.h>			 // include all the good stuff
#include <stdio.h>
#include <math.h>
#include <float.h>
#include <fstream.h>

#include "Interpreter.h"
#include "pac2d.h"
#include "PacEngine.h"
#include "Sound.h"
#include "PacPlus.h"
#include "DipSwitches.h"

// DEFINES ////////////////////////////////////////////////////////////////////

// defines for screen parameters

#define SCREEN_WIDTH     640          // the width of the viewing surface
#define SCREEN_HEIGHT    480          // the height of the viewing surface
#define SCREEN_BPP       8            // the bits per pixel
#define MAX_COLORS       256          // the maximum number of colors

// defines for games

#define NAMCOPAC	1
#define MIDWAYPAC	2
#define MSPACMAN	3
#define PACPLUS		4
#define PACMOD		5
#define PACMANJP	6
#define HANGLY		7
#define PUCKMAN		8
#define MSPACATK	9
#define THEGLOB		10
#define PIRANHA		11
#define CRUSH		12
#define PONPOKO		13

// TYPES /////////////////////////////////////////////////////////////////////

typedef unsigned char  UCHAR;

// MiniPac Class

class MiniPac
{
public:
	MiniPac(HWND hwnd);
	~MiniPac();

	//int GameToPlay;
    int ExecuteFrame(void);
    void Throttle(void);
    void Unthrottle(void);
    void Reset(void);

protected:
    Interpreter			*pacInterpreter;
    Pac2D				*pac2D;
    PacEngine			*pacEngine;
	PacSound			*pacSound;

    LPDIRECTDRAW         ddObject;
    LPDIRECTDRAWSURFACE  ddPrimarySurface;
    LPDIRECTDRAWSURFACE  ddOffscreenSurface;
    LPDIRECTDRAWPALETTE  ddPalette;           		// a pointer to the created dd palette
    PALETTEENTRY         color_palette[256];		// holds the shadow palette entries

	LARGE_INTEGER	timerFreq;
	LARGE_INTEGER	timerValue;
	LARGE_INTEGER	lastTimerValue;
	LARGE_INTEGER	timeTillSecond;
	LARGE_INTEGER	frameFreq;
	LARGE_INTEGER	realFrameFreq;
	LARGE_INTEGER	frameValue;
	LARGE_INTEGER	lastFrameValue;
	LARGE_INTEGER	timeTillFrame;
	long			fps;

    void DDInit(HWND hwnd);
    void DDShutdown(void);
    void CreateSurfaces(void);
    void InitPalette(void);
};

//int GameToPlay;

extern unsigned char PacPlusDecrypt1(unsigned char e);
extern unsigned char PacPlusDecrypt2(unsigned char e);
extern unsigned char PacPlusDecrypt3(unsigned char e);
extern unsigned char PacPlusDecrypt4(unsigned char e);
extern unsigned char PacPlusDecrypt5(unsigned char e);
extern unsigned char PacPlusDecrypt6(unsigned char e);
extern unsigned char PacPlusDecrypt(int addr, unsigned char e);

extern PortData NamcoPacPorts[];
extern DipSwitch NamcoPacSwitches[];
extern DipSwitch MsPacSwitches[];
extern DipSwitch CrushSwitches[];

extern int GameSelected;
extern int DipDefault[10];
extern CODE_ROM NamcoPacRomList[];
extern CODE_ROM MidwayPacRomList[];
extern CODE_ROM MsPacRomList[];
extern CODE_ROM PacPlusRomList[];
extern CODE_ROM PacModRomList[]; 
extern CODE_ROM PacManJpRomList[];
extern CODE_ROM HanglyRomList[];
extern CODE_ROM PuckManRomList[];
extern CODE_ROM MsPacAtkRomList[];
extern CODE_ROM TheGlobRomList[]; 
extern CODE_ROM PiranhaRomList[];
extern CODE_ROM CrushRomList[];
extern CODE_ROM PonpokoRomList[];

